@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2003 PWSCF group
+! Copyright (C) 2001-2016 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -9,10 +9,8 @@
 SUBROUTINE bcast_lr_input
   !-----------------------------------------------------------------------
   !
-  !     The first processor sends the input to all the other processors
+  !  ...The first processor sends the input to all the other processors.
   !
-  !
-  ! Modified by Osman Baris Malcioglu in 2009
 #ifdef __MPI
 
   USE lr_variables
@@ -26,20 +24,17 @@ SUBROUTINE bcast_lr_input
   USE mp_global,           ONLY: intra_image_comm
   USE mp_world,            ONLY: world_comm
   USE exx,                 ONLY: ecutfock
-#ifdef __ENVIRON
-  USE plugin_flags,        ONLY: use_environ
-  USE environ_input,       ONLY: environ_bcast
-#endif
+  USE qpoint,              ONLY: xq
+  USE control_lr,          ONLY: lrpa
 
   IMPLICIT NONE
   !
-  !
-  !
   CALL mp_barrier(world_comm)
   CALL mp_bcast (lr_io_level, ionode_id, world_comm )
   CALL mp_bcast (itermax, ionode_id, world_comm )
   CALL mp_bcast (itermax_int, ionode_id, world_comm )
   CALL mp_bcast (charge_response, ionode_id, world_comm )
+  CALL mp_bcast (sum_rule, ionode_id, world_comm )
   CALL mp_bcast (project, ionode_id, world_comm )
   CALL mp_bcast (restart, ionode_id, world_comm )
   CALL mp_bcast (restart_step, ionode_id, world_comm )
@@ -65,11 +60,18 @@ SUBROUTINE bcast_lr_input
   call mp_bcast (scissor, ionode_id, world_comm)
   CALL mp_bcast (ecutfock, ionode_id, world_comm )
   CALL mp_bcast (d0psi_rs, ionode_id,world_comm )
+  CALL mp_bcast (lshift_d0psi, ionode_id,world_comm )
   CALL mp_bcast (tddfpt, ionode_id, world_comm )
   CALL plugin_arguments_bcast(ionode_id, world_comm)
-#ifdef __ENVIRON
-  IF (use_environ) CALL environ_bcast()
-#endif
+
+  ! for EELS
+  call mp_bcast (eels, ionode_id, world_comm )
+  call mp_bcast (q1, ionode_id, world_comm )
+  call mp_bcast (q2, ionode_id, world_comm )
+  call mp_bcast (q3, ionode_id, world_comm )
+  call mp_bcast (xq, ionode_id, world_comm )
+  call mp_bcast (approximation, ionode_id, world_comm ) 
+  call mp_bcast (lrpa, ionode_id, world_comm ) 
 
   ! for lr_dav
   CALL mp_bcast (davidson, ionode_id, world_comm )
@@ -80,6 +82,7 @@ SUBROUTINE bcast_lr_input
   CALL mp_bcast (precondition, ionode_id, world_comm )
   CALL mp_bcast (dav_debug, ionode_id, world_comm )
   CALL mp_bcast (reference, ionode_id, world_comm )
+  CALL mp_bcast (vccouple_shift, ionode_id, world_comm )
   CALL mp_bcast (single_pole, ionode_id, world_comm )
   CALL mp_bcast (sort_contr, ionode_id, world_comm )
   CALL mp_bcast (diag_of_h, ionode_id, world_comm )
@@ -99,8 +102,9 @@ SUBROUTINE bcast_lr_input
   CALL mp_bcast (max_iter, ionode_id, world_comm )
   CALL mp_bcast (conv_assistant, ionode_id, world_comm )
   CALL mp_bcast (if_dft_spectrum, ionode_id, world_comm )
+  CALL mp_bcast (lplot_drho, ionode_id, world_comm )
   CALL mp_barrier(world_comm)
-
+  
 #endif
   RETURN
 END SUBROUTINE bcast_lr_input
